#!/usr/bin/python
# -*- coding: UTF8 -*-

## Copyright (C) 2007 Red Hat Inc.
## Written by Barry <mbarabas@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import gtk
import re

from datetime import timedelta

try:
    import pango
except:
    print "Cannot load pango :( ... "

import output
from scheduler.scheduler import OutputItem
from scheduler.i18n import _
from scheduler.scheduler import (ST_TestCase, ST_TestResult)

class Options:
    debug = False
    hint = False


class fUtils:
    
    def __init__( self ):

        __ico_path = "/usr/share/pixmaps/sectool/"

        self.markI       = ["<i>","</i>"]
        self.markB       = ["<b>","</b>"]
        self.markU       = ["<u>","</u>"]
        self.markShade   = ["<span color='gray'>","</span>"]
        self.markNone    = ["",""]

        self.wLabel = self.markNone[0] + _("Warning") + self.markNone[1]
        self.eLabel = self.markB[0] + _("Error") + self.markB[1]
        self.hLabel = self.markShade[0] + _("Hint") + self.markShade[1]
        self.iLabel = self.markI[0] + _("Info") + self.markI[1]

        self.headWarning = [gtk.gdk.pixbuf_new_from_file(__ico_path+"warning.png"), self.wLabel]
        self.headError   = [gtk.gdk.pixbuf_new_from_file(__ico_path+"error.png"), self.eLabel]
        self.headInfo    = [gtk.gdk.pixbuf_new_from_file(__ico_path+"info.png"), self.iLabel]
        self.headHint    = [gtk.gdk.pixbuf_new_from_file(__ico_path+"empty.png"), self.hLabel]

        self.headWarningHint = [gtk.gdk.pixbuf_new_from_file(__ico_path+"warning-hint.png"), self.wLabel]
        self.headErrorHint   = [gtk.gdk.pixbuf_new_from_file(__ico_path+"error-hint.png"), self.eLabel]

        self.test_results = { OutputItem.t_error : self.headError,
                              OutputItem.t_warning : self.headWarning, 
                              OutputItem.t_hint : self.headHint,
                              OutputItem.t_info : self.headInfo }
        
        self.test_results_with_hint = { OutputItem.t_error : self.headErrorHint,
                              OutputItem.t_warning : self.headWarningHint, 
                              OutputItem.t_hint : self.headHint,
                              OutputItem.t_info : self.headInfo }

        self.markup = { OutputItem.t_error : self.markB,
                        OutputItem.t_warning : self.markNone,
                        OutputItem.t_hint : self.markShade,
                        OutputItem.t_info : self.markI }
            
    def remove_mark( self, string ):
        
        if string == None: return ""
        str = re.search( ">.*<", string )
         
        if str != None: return str.group(0)[1:-1]
        else:           return string


class OutputFormatter( fUtils ):
    
    def __init__( self):

        fUtils.__init__( self )
        self.hints_dictionary = {}
        self.hint_msg = ""
        self.mainPageCharCounts = 82
        self.stop = False

    def __preprocess_message( self, message ):

        message = " ".join( message.split() )
        message = '\n'.join( message.split( "\\n" ) )
        return message

    def __get_test_result( self, result_number ):
        """ Get result as text by result number"""

        return [
                "<span foreground=\"#51b94a\"><b>"+_("PASSED")+"</b></span>",
                "<span foreground=\"#d5b92c\"><b>"+_("WARNING")+"</b></span>",
                "<span foreground=\"red\"><b>"+_("ERROR")+"</b></span>",
                "<span foreground=\"red\"><b>"+_("FAILURE !")+"</b></span>",
                "<span foreground=\"red\"><b>"+_("FATAL")+"</b></span>",
                ][result_number]

    def __add_hint_to_dictionary( self, test_name, hint_number, message ):
        """ Add hint to dictionary """
        #: remove long spaces and preprocessing the message
        message = self.__preprocess_message( message )
        if not test_name in self.hints_dictionary:
            self.hints_dictionary[test_name] = {hint_number: message}
        else: self.hints_dictionary[test_name][hint_number] = message

    def __get_hint( self, test, msg_id ):
        
        test = self.remove_mark( test )
        msg_id = self.remove_mark( msg_id )
        if test in self.hints_dictionary and msg_id in self.hints_dictionary[test]:
            return self.hints_dictionary[test][msg_id]
        else: return ""

    def init_handlers( self, renderer ):
        """ Initialize signal handler object for all signals from widgets """

        self.renderer = renderer
        self.testResults = None

    def renderApp( self, window ):
        """ 
        Main function of rendering main application window.
        Calling Renderer's methods to render GTK objects

        Setting gray color of background default level in testView
        """

        rootBox = self.renderer.RootBox( window )  # Base box
        self.renderer.MenuBar( rootBox )                    # Top menu bar
        self.renderer.ToolBar( rootBox )                    # Tool bar below menu bar
        # render boxes for menu and mainView
        (menuBox, mainBox) = self.renderer.MainPane( rootBox, (5, 5, 5, 5) ) # Box with left "menu" and right "mainBox"

        #render left part of mainView
        self.renderer.SimpleFilter( menuBox )                 # Test filter box with group filter and level filter
        self.renderer.TestView( menuBox )                     # Box with list of tests

        # render right part of mainView
        self.renderer.MainBook( mainBox )                     # MainBook with list of messages

        # render status bar at the bototm of main window
        self.renderer.StatusBar( rootBox )                     # here comes the StatusBar with progress bar (of running tests)

        # Box for information about selected message
        self.renderer.TestInfoBox( window )

        # Set color of selected level on the list of tests 
        self.renderer.OTestList.set_level( self.renderer.group_filter.get_active()+1 )

    def show_progress( self ):
        """ Just show the progressbar inside the status bar """

        self.renderer.OStatusWindow.show( )

    def clear_results( self ):
        """ Clear all! New test suite is going to be executed """

        self.testResults = {}
        self.timeResults = {}

    def save_to_file( self ):
        """ Save result to the file """

        # padding for listing results
        w_label = 12
        w_id    = 6
        w_total = 80

        if self.testResults != None and len( self.testResults.keys() ) > 0:

            str_test_results = [ _("PASS"), _("WARNING"), _("ERROR"), _("FAIL"), _("FATAL")] 
            str_labels = { OutputItem.t_error : _("Error"),
                        OutputItem.t_warning : _("Warning"), 
                        OutputItem.t_hint : _("Hint"),
                        OutputItem.t_info : _("Info") }

            options = Options()
            options.hint = self.renderer.OMenuBar.get_hint_active()
            options.debug = self.renderer.OMenuBar.get_info_active() 
            formatter = output.StdoutFormatter()
            pathname = self.renderer.OpenDialog()
            #: open file
            if pathname == None: return
            File = open( pathname, "w" )

            for test in self.testResults.keys():
                result = self.testResults[test]
                test_result = str_test_results[result.result]
                header = _("Test Name:")+" %s" % (test)
                str_result = (_("Test Result:")+" %s" % (test_result)).rjust(w_total - len(header))
                header = "%s%s" % (header, str_result)
                File.writelines( header+'\n' )

                # regular test output
                for item in result.output_items:
                    # don't print hints without the proper option
                    if item.type == OutputItem.t_hint and options.hint == False:
                        continue

                    # debug messages are printed without the id fiels
                    if item.type == OutputItem.t_info:
                        if options.debug == True:
                            msg = ("\t%s" % (str_labels[item.type])).ljust(15)
                        else:
                            continue
                    else:
                        msg = ("\t%s(%s)" % (str_labels[item.type], item.msg_id)).ljust(15)

                    # add the actual payload and print it
                    msg += item.msg.strip()
                    File.writelines( msg+'\n' )

            File.close( )
            mbox = gtk.MessageDialog( None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Results saved to:")+"\n\t"+pathname )
            mbox.run( )
            mbox.destroy( )

        else:
            mbox = gtk.MessageDialog( None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("No results. Please run the test at first !") )
            mbox.run( )
            mbox.destroy( )

    def showTest( self, test ):

        test_name = test.dsc['HEADER']['NAME']
        self.iter = self.renderer.OOutputTestList.append([test_name, "<i>testing...</i>"])
        self.old_item = None
        self.timeResults[test_name] = 0

    def showTestResult( self, test, result, time ):

        test_name = test.dsc['HEADER']['NAME']
        # save results
        self.timeResults[test_name] = time
        
        if self.old_item != None: 
            self.old_item.msg = self.old_item.msg.strip()
            message = self.test_results[self.old_item.type]
            if self.old_item.msg_id == None: self.old_item.msg_id = ""
            message = message + [self.markup[self.old_item.type][0]+test.dsc['HEADER']['NAME']+self.markup[self.old_item.type][1]] \
                            + [self.markup[self.old_item.type][0]+self.old_item.msg_id+self.markup[self.old_item.type][1]]  \
                            + [self.markup[self.old_item.type][0]+self.old_item.msg+self.markup[self.old_item.type][1]] 
            try:
                #self.renderer.appendMessage( self.renderer.outputView.get_model().get_model(), message )
                self.renderer.OOutputTestResults.append_message( message )
            except Exception, e:
                print message
                print e
        result_message = self.__get_test_result( result.result )
        self.testResults[test_name] = result
        self.renderer.OOutputTestList.set_value( self.iter, 1, result_message )

    def showTestLine( self, test=None, item=None ):
        """ Show messages from test result """    

        test_name = test.dsc['HEADER']['NAME']

        #for i, item in enumerate( result.output_items ):
        message = []
        if item.type == OutputItem.t_hint:
            self.__add_hint_to_dictionary( test.dsc['HEADER']['NAME'], item.msg_id, item.msg.strip() )
            if self.old_item != None: 
                message = self.test_results_with_hint[self.old_item.type]
                message = message + [self.markup[self.old_item.type][0]+test.dsc['HEADER']['NAME']+self.markup[self.old_item.type][1]] \
                                + [self.markup[self.old_item.type][0]+self.old_item.msg_id+self.markup[self.old_item.type][1]]  \
                                + [self.markup[self.old_item.type][0]+self.old_item.msg+self.markup[self.old_item.type][1]] 
                self.old_item = None
            else: raise ImplementError
        elif self.old_item != None:
            self.old_item.msg = self.old_item.msg.strip()
            message = self.test_results[self.old_item.type]
            if self.old_item.msg_id == None: self.old_item.msg_id = ""
            message = message + [self.markup[self.old_item.type][0]+test.dsc['HEADER']['NAME']+self.markup[self.old_item.type][1]] \
                            + [self.markup[self.old_item.type][0]+self.old_item.msg_id+self.markup[self.old_item.type][1]]  \
                            + [self.markup[self.old_item.type][0]+self.old_item.msg+self.markup[self.old_item.type][1]] 

            self.old_item = item

        else: 
            self.old_item = item
            return

        try:
            #self.renderer.appendMessage( self.renderer.outputView.get_model().get_model(), message )
            self.renderer.OOutputTestResults.append_message( message )
        except Exception, e:
            print message
            print e


    def show_test_summary( self, test ):
        """ Make a summary of test on the right of result screen """

        messages = errors = warnings = hints = info = 0

        if test in self.testResults: #: for one specific test
            messages = len( self.testResults[test].output_items )
            for item in self.testResults[test].output_items:
                if item.type == OutputItem.t_error: errors += 1
                elif item.type == OutputItem.t_warning: warnings += 1
                elif item.type == OutputItem.t_hint: hints += 1
                elif item.type == OutputItem.t_info: info += 1

            if test in self.timeResults: time = timedelta( seconds = self.timeResults[test] )
            else: time = timedelta( seconds = 0 )
            self.renderer.OOutputTestSummary.set_text( _('Messages: ')+`messages`+'\n'+
                                                      _('Errors: ')+`errors`+'\n'+
                                                      _('Warnings: ')+`warnings`+'\n'+
                                                      _('Hints: ')+`hints`+'\n'+
                                                      _('Info messages: ')+`info`+'\n'+
                                                      '\n\n'+
                                                      _('Time the test was running: ')+`time.seconds/60`+'m '+`time.seconds%60`+'s '+'\n'
                                                      )
            if self.testResults[test].result == 3 and self.stop: 
                self.renderer.OOutputTestSummary.set_text( self.renderer.OOutputTestSummary.get_text() + '\n\n<b>' +
                        _('Test failed: User pressed stop button. Please restart this test to get all results.') + '</b>\n\n' )
                
        elif test != None and test not in self.timeResults: #: for all tests ( the first line "All tests" )
            for test in self.testResults.keys():
                messages += len( self.testResults[test].output_items )
                for item in self.testResults[test].output_items:
                    if item.type == OutputItem.t_error: errors += 1
                    elif item.type == OutputItem.t_warning: warnings += 1
                    elif item.type == OutputItem.t_hint: hints += 1
                    elif item.type == OutputItem.t_info: info += 1
            time = 0
            for item in self.timeResults.keys():
                time += float(self.timeResults[item])
            d = timedelta( seconds = time )
            
            self.renderer.OOutputTestSummary.set_text( _('Messages: ')+`messages`+'\n'+
                                                      _('Errors: ')+`errors`+'\n'+
                                                      _('Warnings: ')+`warnings`+'\n'+
                                                      _('Hints: ')+`hints`+'\n'+
                                                      _('Info messages: ')+`info`+'\n'+
                                                      '\n\n'+
                                                      _('Summary of time: ')+`d.seconds/60`+'m '+`d.seconds%60`+'s '+'\n'
                                                      )
        else: # test not in testResults, but in timeResults, so the test is in the testing status right now
            self.renderer.OOutputTestSummary.set_text( _('Messages: ')+'-'+'\n'+
                                                      _('Errors: ')+'-'+'\n'+
                                                      _('Warnings: ')+'-'+'\n'+
                                                      _('Hints: ')+'-'+'\n'+
                                                      _('Info messages: ')+'-'+'\n'+
                                                      '\n\n'+
                                                      _('Summary of time: ')+_('Unknown')+'\n'
                                                      )

    def BtnEvent_info_toggle( self, active ):
        """ Info button toggled, make info buttons (in)active """
        self.renderer.OToolBar.set_info_active( active )
        self.renderer.OMenuBar.set_info_active( active )

    def BtnEvent_hint_toggle( self, active ):
        """ Hint button toggled, make hint buttons (in)active """
        self.renderer.OToolBar.set_hint_active( active )
        self.renderer.OMenuBar.set_hint_active( active )

    def BtnEvent_stop_pressed( self ):
        """ Stop button pressed. The last test could end with "FAILED". Make an explanation """
        self.stop = True

    def refilter_messages( self ):
        """ Refilter the message window. Visibility of hints or info messages have changed """
        
        (model, iter) = self.renderer.OOutputTestList.get_selected()
        
        if iter != None:
            filter_all = self.remove_mark( model.get_value( iter, 0 ) ) == _("All")
            test_name  = model.get_value( iter, 0 )
        else: 
            filter_all = True
            test_name = None
        info = self.renderer.OMenuBar.get_info_active()
        
        self.renderer.OOutputTestResults.refilter( iter, filter_all, test_name, info )

    def clear_output( self ):
        """ Clear all - new instance of testing is being started """
        self.hits_dictionary = {}
        self.hint_msg = ""
        #self.renderer.outputView.get_model().get_model().get_model().clear( )
        self.renderer.OOutputTestResults.clear_model();
        self.stop = False
        self.renderer.OOutputTestSummary.set_text( " " )
        self.renderer.OOutputTestList.clear()
        self.renderer.OOutputTestList.append( ["<i>"+_("All")+"</i>", "<i>"+_("Show all test results")+"</i>"] )

    def show_message_hint( self, test_name, msg_id ):
        """ Show message hint in small yellow box """

        self.hint_msg = self.remove_mark( self.__get_hint( test_name, msg_id ) )
        if len( self.hint_msg ) > 0: 
            self.renderer.hintLabel.set_text( self.hint_msg )
            if self.renderer.OMenuBar.get_hint_active(): self.renderer.hintWindow.show_all( )
            else: self.renderer.hintWindow.hide( )
        else: self.renderer.hintWindow.hide( )

    def show_test_info( self, name, author, description, groups, levels, missing_reqs ):
        """ Show info about test choosen from test list on the main page """
        
        CHAR_NUM = self.mainPageCharCounts

        self.renderer.testInfo.set_text( _('Name: ')+name+'\n'+_('Author: ')+author )
        lines = description.split('\n')
        #lines_count = len(lines)
        description = ""
        for line in lines: 
            if line[0] == '*': 
                description += '    -'
                description += line[1:]+'\n'

            elif len( re.sub(r'\s', '', line) ) == 0 or line[0:4] == "--":
                description += '\n'
    
            elif line[0:4] == "----":
                description += ''.join(['-' for i in range(int(CHAR_NUM)) ]) +'\n'
                #description += ''+'\n'

            else: description += line+'\n'

        self.renderer.testDescription.set_text( description )
        self.renderer.testGroups.set_text( groups )
        __level2text = ('', _("Naive"), _("Desktop"), _("Network"), _("Server"), _("Paranoid"))
        __levels = ""
        for l in levels:
            if l != 0: 
                __levels += __level2text[ l ]
                __levels += ", "
        __levels = __levels[:-2]
        self.renderer.testLevels.set_text( __levels )

        if len( missing_reqs ) != 0: self.renderer.OStatusWindow.push_requirements( missing_reqs )
        else: self.renderer.OStatusWindow.pop( 1 )
        
    def updateProgressBar( self, progress=None, label=None ):
        """ Update progress and label in progressBar """

        self.renderer.OStatusWindow.update_progress( progress, label )
        if progress == 1.0: self.renderer.mainBook.set_current_page( 1 )
        
    def hide_progressBar( self ):
        """ If progressButton is not active hide progressBar - tests are done """
        if not self.renderer.OStatusWindow.pb_is_active( ) and self.renderer.OStatusWindow.get_fraction() == 1.0: 
            self.renderer.OStatusWindow.hide()
    
    def clear_testView( self ):
        """ We are going to make a new list of tests - clear current """
        self.renderer.OTestList.clear( )
        
    def get_selected_tests( self ):
        """ Get list of selected tests """
        return self.renderer.OTestList.get_selected_tests( )
    
    def highlight_page( self, page, highlight ):
        """ Something is changed in the notebook pages .. make it bold or unbold """
        label = self.renderer.mainBook.get_tab_label( self.renderer.mainBook.get_nth_page( page ) )
        if highlight == True: 
            label.set_markup( "<b>"+label.get_text()+"</b>" )    
        else: 
            label.set_markup( self.remove_mark( label.get_text( ) ) )

    def highlight_level( self, column ):
        """ Level is changed .. make appropriate column gray """
        self.renderer.OTestList.set_level( column )
    
    def select_level( self, level ):

        self.renderer.level_filter.set_active( int(level) - 1 )

    def MainWindow_resized( self, x, y ):
        """ Resize all objects depending on window size """
        self.mainPageCharCounts = 82 + (x-800)/6
        size = self.mainPageCharCounts

        for separator in self.renderer.mainPageSeparators:
            separator.set_size_request( size*6+20, 5 )
        for separator in self.renderer.resultPageSeparators:
            separator.set_size_request( size*3, 5 )
        self.renderer.testDescription.set_size_request( size*6, -1  )

    def hintWindow_move( self, x, y ):
        """ Move with hint window """
        self.renderer.hintWindow.move( x, y )

    def hintWindow_hide( self ):
        """ Hide hint window """
        if self.renderer.hintWindow != None: self.renderer.hintWindow.hide( )

    def hintWindow_show( self ):
        """ Show hint window """
        if self.renderer.hintWindow != None: self.renderer.hintWindow.show_all( )

    def add_test( self, test, missing_reqs ):
        """ Add test to the list """
        if len( missing_reqs ) == 0: self.renderer.OTestList.append_test( test )
        else:
            test[-6:-1] = [False, False, False, False, False]
            test[0] = "<span foreground='gray'>"+test[0]+"</span>"
            self.renderer.OTestList.append_test( test )
        
    def add_group( self, group ):
        """ Add group to the group-filter """
        self.renderer.group_filter.append_text( group )
        
    def get_active_level( self ):
        """ Get active level """
        return self.renderer.level_filter.get_active( )+1
    
    def get_active_tests(self, level):
        """ Get all tests active in level """
        return self.renderer.OTestList.get_active_tests( level )

    def get_tests_levels( self ):
        """ Get all levels for all tests """
        return self.renderer.OTestList.get_test_levels( ) 
    
    def save_box( self ):

        mbox = gtk.MessageDialog( None, 0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, _("Configuration of test levels has changed.\n\nDo you want to save new configuration?") )
        result = mbox.run( )
        mbox.destroy( )

        return result == -8

    def set_test_levels( self, test, levels ):

        self.renderer.OTestList.set_test_levels( test, levels )

    def set_btn_sensitivity( self, sensitive ):
        """ Set sensitivity of buttons depending on test executing process """
        self.renderer.OToolBar.set_sensitive( sensitive )
        self.renderer.OTestList.set_popup_sensitivity( not sensitive )
        self.renderer.OMenuBar.set_sensitivity( not sensitive )

    def select_test( self, number ):
        """ Select test in 'number' position in testView """
        # TODO: check if there is some 
        self.renderer.OTestList.select_test( number )
